'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});
exports.Fetch = exports.FetchFunctions = undefined;

var _bluebird = require('bluebird');

var _bluebird2 = _interopRequireDefault(_bluebird);

var _isomorphicFetch = require('isomorphic-fetch');

var _isomorphicFetch2 = _interopRequireDefault(_isomorphicFetch);

var _jwt = require('./jwt');

var _jwt2 = _interopRequireDefault(_jwt);

var _utils = require('./utils');

var _utils2 = _interopRequireDefault(_utils);

var _config = require('./config');

var _config2 = _interopRequireDefault(_config);

var _dedupeCalls = require('./utils/dedupe-calls');

var _dedupeCalls2 = _interopRequireDefault(_dedupeCalls);

var _urlconfig = require('./urlconfig');

var _urlconfig2 = _interopRequireDefault(_urlconfig);

var _scopes = require('./scopes');

var _LoadingIndicator = require('./LoadingIndicator');

var _LoadingIndicator2 = _interopRequireDefault(_LoadingIndicator);

var _index = require('./capability/index');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var refreshToken = null;

function isGetRequest(fetchOptions) {
    return !fetchOptions || !fetchOptions.method || 'get'.localeCompare(fetchOptions.method) === 0;
}

var FetchFunctions = exports.FetchFunctions = {
    /**
     * Ensures the URL starts with jenkins path if not an absolute URL.
     * @param url
     * @returns {string}
     */
    prefixUrl: function prefixUrl(url) {
        if (url.indexOf('http') === 0) {
            return url;
        }

        if (_urlconfig2.default.getJenkinsRootURL() !== '' && !url.startsWith(_urlconfig2.default.getJenkinsRootURL())) {
            return '' + _urlconfig2.default.getJenkinsRootURL() + url;
        }

        return url;
    },
    checkRefreshHeader: function checkRefreshHeader(response) {
        var _refreshToken = response.headers.get('X-Blueocean-Refresher');
        // No token in response, lets just ignore.
        if (!_refreshToken) {
            return response;
        }

        // First time we have seen a refresh token, early exit.
        if (!refreshToken) {
            refreshToken = _refreshToken;
            return response;
        }

        // We need to refresh the page now!
        if (refreshToken !== _refreshToken) {
            _utils2.default.refreshPage();
            throw new Error('refreshing apge');
        }
        return response;
    },


    /**
     * This method checks for for 2XX http codes. Throws error it it is not.
     * This should only be used if not using fetch or fetchJson.
     */
    checkStatus: function checkStatus(response) {
        if (response.status >= 300 || response.status < 200) {
            var message = 'fetch failed: ' + response.status + ' for ' + response.url;
            var error = new Error(message);
            error.response = response;
            throw error;
        }
        return response;
    },
    stopLoadingIndicator: function stopLoadingIndicator(response) {
        _LoadingIndicator2.default.hide();
        return response;
    },


    /**
     * Adds same-origin option to the fetch.
     */
    sameOriginFetchOption: function sameOriginFetchOption() {
        var options = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};

        var newOpts = _utils2.default.clone(options);
        newOpts.credentials = newOpts.credentials || 'same-origin';
        return newOpts;
    },


    /**
     * Enhances the fetchOptions with the JWT bearer token. Will only be needed
     * if not using fetch or fetchJson.
     */
    jwtFetchOption: function jwtFetchOption(token) {
        var options = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};

        var newOpts = _utils2.default.clone(options);
        newOpts.headers = newOpts.headers || {};
        newOpts.headers.Authorization = newOpts.headers.Authorization || 'Bearer ' + token;
        return newOpts;
    },


    /**
     * REturns the json body from the response. It is only needed if
     * you are using FetchUtils.fetch
     *
     * Usage:
     * FetchUtils.fetch(..).then(FetchUtils.parseJSON)
     */
    parseJSON: function parseJSON(response) {
        return response.json()
        // FIXME: workaround for status=200 w/ empty response body that causes error in Chrome
        // server should probably return HTTP 204 instead
        .catch(function (error) {
            if (error.message.indexOf('Unexpected end of JSON input') !== -1) {
                return {};
            }
            throw error;
        });
    },


    /* eslint-disable no-param-reassign */
    /**
     * Parses the response body for the error generated in checkStatus.
     */
    parseErrorJson: function parseErrorJson(error) {
        return error.response.json().then(function (body) {
            error.responseBody = body;
            throw error;
        }, function () {
            error.responseBody = null;
            throw error;
        });
    },

    /* eslint-enable no-param-reassign */

    /**
    * Error function helper to log errors to console.
    *
    * Usage;
    * fetchJson(..).catch(FetchUtils.consoleError)
    */
    consoleError: function consoleError(error) {
        console.error(error); // eslint-disable-line no-console
    },


    /**
     * Error function helper to call a callback on a rejected promise.
     * if callback is null, log to console). Use .catch() if you know it
     * will not be null though.
     *
     * Usage;
     * fetchJson(..).catch(FetchUtils.onError(error => //do something)
     */
    onError: function onError(errorFunc) {
        return function (error) {
            if (errorFunc) {
                errorFunc(error);
            } else {
                FetchFunctions.consoleError(error);
            }
        };
    },


    /**
    * Raw fetch that returns the json body.
    *
    * This method is semi-private, under normal conditions it should not be
    * used as it does not include the JWT bearer token
    *
    * @param {string} url - The URL to fetch from.
    * @param {Object} [options]
    * @param {function} [options.onSuccess] - Optional callback success function.
    * @param {function} [options.onError] - Optional error callback.
    * @param {Object} [options.fetchOptions] - Optional isomorphic-fetch options.
    * @param {boolean} [options.disableDedupe] - Optional flag to disable dedupe for this request.
    * @param {boolean} [options.disableLoadingIndicator] - Optional flag to disable loading indicator for this request.
    * @returns JSON body
    */
    rawFetchJSON: function rawFetchJSON(url) {
        var _ref = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {},
            onSuccess = _ref.onSuccess,
            onError = _ref.onError,
            fetchOptions = _ref.fetchOptions,
            disableDedupe = _ref.disableDedupe,
            disableLoadingIndicator = _ref.disableLoadingIndicator,
            ignoreRefreshHeader = _ref.ignoreRefreshHeader;

        var request = function request() {
            var future = getPrefetchedDataFuture(url); // eslint-disable-line no-use-before-define

            if (!disableLoadingIndicator) {
                _LoadingIndicator2.default.show();
            }

            if (!future) {
                future = (0, _isomorphicFetch2.default)(url, FetchFunctions.sameOriginFetchOption(fetchOptions));

                if (!ignoreRefreshHeader) {
                    future = future.then(FetchFunctions.checkRefreshHeader);
                }

                future = future.then(FetchFunctions.checkStatus).then(FetchFunctions.parseJSON, FetchFunctions.parseErrorJson);

                if (!disableLoadingIndicator) {
                    future = future.then(FetchFunctions.stopLoadingIndicator, function (err) {
                        FetchFunctions.stopLoadingIndicator();throw err;
                    });
                }
            } else if (!disableLoadingIndicator) {
                _LoadingIndicator2.default.hide();
            }
            if (onSuccess) {
                return future.then(onSuccess).catch(FetchFunctions.onError(onError));
            }

            return future;
        };
        if (disableDedupe || !isGetRequest(fetchOptions)) {
            return request();
        }

        return (0, _dedupeCalls2.default)(url, request);
    },

    /**
     * Raw fetch.
     *
     * This method is semi-private, under normal conditions it should not be
     * used as it does not include the JWT bearer token
     *
     * @param {string} url - The URL to fetch from.
     * @param {Object} [options]
     * @param {function} [options.onSuccess] - Optional callback success function.
     * @param {function} [options.onError] - Optional error callback.
     * @param {Object} [options.fetchOptions] - Optional isomorphic-fetch options.
     * @param {boolean} [options.disableDedupe] - Optional flag to disable dedupe for this request.
     * @param {boolean} [options.disableLoadingIndicator] - Optional flag to disable loading indicator for this request.
     * @returns fetch response
     */
    rawFetch: function rawFetch(url) {
        var _ref2 = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {},
            onSuccess = _ref2.onSuccess,
            onError = _ref2.onError,
            fetchOptions = _ref2.fetchOptions,
            disableDedupe = _ref2.disableDedupe,
            disableLoadingIndicator = _ref2.disableLoadingIndicator,
            ignoreRefreshHeader = _ref2.ignoreRefreshHeader;

        var request = function request() {
            var future = getPrefetchedDataFuture(url); // eslint-disable-line no-use-before-define
            if (!future) {
                if (!disableLoadingIndicator) {
                    _LoadingIndicator2.default.show();
                }

                future = (0, _isomorphicFetch2.default)(url, FetchFunctions.sameOriginFetchOption(fetchOptions));

                if (!ignoreRefreshHeader) {
                    future = future.then(FetchFunctions.checkRefreshHeader);
                }

                future = future.then(FetchFunctions.checkStatus);

                if (!disableLoadingIndicator) {
                    future = future.then(FetchFunctions.stopLoadingIndicator, function (err) {
                        FetchFunctions.stopLoadingIndicator();throw err;
                    });
                }
            }
            if (onSuccess) {
                return future.then(onSuccess).catch(FetchFunctions.onError(onError));
            }
            return future;
        };

        if (disableDedupe || !isGetRequest(fetchOptions)) {
            return request();
        }

        return (0, _dedupeCalls2.default)(url, request);
    }
};

var Fetch = exports.Fetch = {
    /**
     * Fetch JSON data.
     * <p>
     * Utility function that can be mocked for testing.
     *
     * @param {string} url - The URL to fetch from.
     * @param {Object} [options]
     * @param {function} [options.onSuccess] - Optional callback success function.
     * @param {function} [options.onError] - Optional error callback.
     * @param {Object} [options.fetchOptions] - Optional isomorphic-fetch options.
     * @returns JSON body.
     */
    fetchJSON: function fetchJSON(url) {
        var _ref3 = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {},
            onSuccess = _ref3.onSuccess,
            onError = _ref3.onError,
            fetchOptions = _ref3.fetchOptions,
            disableCapabilites = _ref3.disableCapabilites,
            disableLoadingIndicator = _ref3.disableLoadingIndicator,
            ignoreRefreshHeader = _ref3.ignoreRefreshHeader;

        var fixedUrl = FetchFunctions.prefixUrl(url);
        var future = void 0;
        if (!_config2.default.isJWTEnabled()) {
            future = FetchFunctions.rawFetchJSON(fixedUrl, { onSuccess: onSuccess, onError: onError, fetchOptions: fetchOptions, disableLoadingIndicator: disableLoadingIndicator, ignoreRefreshHeader: ignoreRefreshHeader });
        } else {
            future = _jwt2.default.getToken().then(function (token) {
                return FetchFunctions.rawFetchJSON(fixedUrl, {
                    onSuccess: onSuccess,
                    onError: onError,
                    fetchOptions: FetchFunctions.jwtFetchOption(token, fetchOptions)
                });
            });
        }

        if (!disableCapabilites) {
            return future.then(function (data) {
                return _index.capabilityAugmenter.augmentCapabilities(_utils2.default.clone(data));
            });
        }

        return future;
    },


    /**
     * Fetch data.
     * <p>
     * Utility function that can be mocked for testing.
     *
     * @param {string} url - The URL to fetch from.
     * @param {Object} [options]
     * @param {function} [options.onSuccess] - Optional callback success function.
     * @param {function} [options.onError] - Optional error callback.
     * @param {Object} [options.fetchOptions] - Optional isomorphic-fetch options.
     * @returns fetch body.
     */
    fetch: function fetch(url) {
        var _ref4 = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {},
            onSuccess = _ref4.onSuccess,
            onError = _ref4.onError,
            fetchOptions = _ref4.fetchOptions,
            disableLoadingIndicator = _ref4.disableLoadingIndicator,
            ignoreRefreshHeader = _ref4.ignoreRefreshHeader;

        var fixedUrl = FetchFunctions.prefixUrl(url);

        if (!_config2.default.isJWTEnabled()) {
            return FetchFunctions.rawFetch(fixedUrl, { onSuccess: onSuccess, onError: onError, fetchOptions: fetchOptions, disableLoadingIndicator: disableLoadingIndicator, ignoreRefreshHeader: ignoreRefreshHeader });
        }

        return _jwt2.default.getToken().then(function (token) {
            return FetchFunctions.rawFetch(fixedUrl, {
                onSuccess: onSuccess,
                onError: onError,
                fetchOptions: FetchFunctions.jwtFetchOption(token, fetchOptions)
            });
        });
    }
};

function trimRestUrl(url) {
    var REST_PREFIX = 'blue/rest/';
    var prefixOffset = url.indexOf(REST_PREFIX);

    if (prefixOffset !== -1) {
        return url.substring(prefixOffset);
    }

    return url;
}

function getPrefetchedDataFuture(url) {
    var trimmedUrl = trimRestUrl(url);

    for (var prop in _scopes.prefetchdata) {
        if (_scopes.prefetchdata.hasOwnProperty(prop)) {
            var preFetchEntry = _scopes.prefetchdata[prop];
            if (preFetchEntry.restUrl && preFetchEntry.data) {
                // If the trimmed/normalized rest URL matches the url arg supplied
                // to the function, construct a pre-resolved future object containing
                // the prefetched data as the value.
                if (trimRestUrl(preFetchEntry.restUrl) === trimmedUrl) {
                    try {
                        return _bluebird2.default.resolve(JSON.parse(preFetchEntry.data));
                    } finally {
                        // Delete the preFetchEntry i.e. we only use these entries once. So, this
                        // works only for the first request for the data at that URL. Subsequent
                        // calls on that REST endpoint will result in a proper fetch. A local
                        // store needs to be used (redux/mobx etc) if you want to avoid multiple calls
                        // for the same data. This is not a caching layer/mechanism !!!
                        delete _scopes.prefetchdata[prop];
                    }
                }
            }
        }
    }

    return undefined;
}
//# sourceMappingURL=fetch.js.map
